#!/bin/sh
# See https://mariadb.com/kb/en/mariadb/documentation/sql-commands/administration-commands/flush-commands/flush/
# and http://dba.stackexchange.com/questions/19532/safest-way-to-perform-mysqldump-on-a-live-system-with-active-reads-and-writes
# and http://stackoverflow.com/questions/20025209/attach-to-mysql-client-entirely-via-fifos

# Author: Adrian Kousz <ak@ivaris.ch>, 2014

# WARNING: no spaces in path names
BKP_FILES='/data/www /data/ssl /etc'
DB_NAMES='pkorg_live mysql'
MOUNTPOINT='/tmp/backup'
OUTDIR="$MOUNTPOINT/$(hostname).ivaris.ch"
MOUNTTRIES=3



echo -n 'Backup started: '
date -Ins

# Mounting
i=$MOUNTTRIES
while [ $((i--)) -gt 0 ]; do
	$(dirname $0)/mountbackup.sh "$MOUNTPOINT"
	if [ $? -eq 0 ]; then break
	else echo 'Mount problem. Retry...' >&2
	fi
done
if [ ! -d "$OUTDIR" ]; then
	echo "$OUTDIR does not exist. Abort!" >&2
	exit 1
fi
####

mkdir -p "$OUTDIR/DB"



# Filename Vars
	if [ $(date +%u) -eq 7 ]; then
		datum="woche_$(date +%W)"
		rm "$OUTDIR"/woche*
		rm "$OUTDIR"/DB/woche*
	else
		datum="tag_$(date +%m%d)"
		rm "$OUTDIR"/tag*
		rm "$OUTDIR"/DB/tag*
	fi
####



# Backup DB
echo -n 'DB Backup started: '
date -Ins

CMD='FLUSH TABLES WITH READ LOCK AND DISABLE CHECKPOINT;'
LOCKSEC=300

# Setup lock
mysql -Ae "$CMD" # Blocking
sync
sleep $LOCKSEC | mysql -A & PID=$!
echo "$CMD" > /proc/$PID/fd/0
####

for db in $DB_NAMES; do
	mysqldump --skip-lock-tables --hex-blob $db | gzip -4 > "$OUTDIR/DB/$datum-$db.sql.gz"
done

# Release lock
echo exit > /proc/$PID/fd/0
####



# Backup files
echo -n 'File Backup started: '
date -Ins

for file in $BKP_FILES; do
	tar -c $file | gzip -4 > "$OUTDIR/$datum-$(basename $file).tar.gz"
done
####



echo -n 'Backup finished: '
date -Ins

umount "$MOUNTPOINT"
